package org.apfloat.internal;

import org.apfloat.*;

import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 * @since 1.1
 * @version 1.1
 * @author Mikko Tommila
 */

public class ParallelRunnerTest
    extends TestCase
{
    public ParallelRunnerTest(String methodName)
    {
        super(methodName);
    }

    public static void main(String[] args)
    {
        junit.textui.TestRunner.run(suite());
    }

    public static TestSuite suite()
    {
        TestSuite suite = new TestSuite();

        suite.addTest(new ParallelRunnerTest("testRunParallel"));

        return suite;
    }

    public static void testRunParallel()
    {
        for (int threads = 1; threads <= 32; threads++)
        {
            ApfloatContext.getContext().setNumberOfProcessors(threads);
            final int LENGTH = 1000000;
            final boolean[] values = new boolean[LENGTH];
            ParallelRunnable parallelRunnable = new ParallelRunnable()
            {
                public int getLength()
                {
                    return LENGTH;
                }

                public Runnable getRunnable(final int start, final int length)
                {
                    return new Runnable()
                    {
                        public void run()
                        {
                            for (int i = start; i < start + length; i++)
                            {
                                values[i] = true;
                            }
                        }
                    };
                }
            };

            ParallelRunner.runParallel(parallelRunnable);

            for (int i = 0; i < LENGTH; i++)
            {
                assertTrue(threads + " threads, element " + i, values[i]);
            }
        }
    }
}
